@@ -1,8 +1,7 @@
 cp ../../pngminus/pnm2png.c pnm2pngm.c
 cp ../../../*.h .
 cp ../../../*.c .
-rm pnggccrd.c pngvcrd.c
-rm example.c pngtest.c pngr*.c pngpread.c
+rm -f example.c pngtest.c pngpread.c pngr*.c pnggccrd.c pngvcrd.c
 # Change the next 2 lines if zlib is somewhere else.
 cp ../../../../zlib/*.h .
 cp ../../../../zlib/*.c .
